import os

from flask import url_for
from invenio_records.api import Record
from oarepo_communities.converters import CommunityPIDValue
from oarepo_communities.proxies import current_oarepo_communities
from oarepo_communities.record import CommunityRecordMixin
from oarepo_records_draft.record import InvalidRecordAllowedMixin, DraftRecordMixin
from oarepo_references.mixins import ReferenceEnabledRecordMixin
from oarepo_validate import SchemaKeepingRecordMixin, MarshmallowValidatedRecordMixin

from nr_nresults.constants import NRESULTS_ALLOWED_SCHEMAS, NRESULTS_PREFERRED_SCHEMA, published_index_name, \
    draft_index_name, all_nresults_index_name
from nr_nresults.marshmallow import NResultsMetadataSchemaV1


class NResultBaseRecord(SchemaKeepingRecordMixin,
                        MarshmallowValidatedRecordMixin,
                        ReferenceEnabledRecordMixin,
                        CommunityRecordMixin,
                        Record,
                        ):
    ALLOWED_SCHEMAS = NRESULTS_ALLOWED_SCHEMAS
    PREFERRED_SCHEMA = NRESULTS_PREFERRED_SCHEMA
    MARSHMALLOW_SCHEMA = NResultsMetadataSchemaV1


class PublishedNResultRecord(InvalidRecordAllowedMixin, NResultBaseRecord):
    index_name = published_index_name

    @property
    def canonical_url(self):
        return url_for('invenio_records_rest.nresults-community_item',
                       pid_value=CommunityPIDValue(
                           self['control_number'],
                           current_oarepo_communities.get_primary_community_field(self)),
                       _external=True)


class DraftNResultRecord(DraftRecordMixin, NResultBaseRecord):
    index_name = draft_index_name

    @property
    def canonical_url(self):
        return url_for('invenio_records_rest.draft-nresults-community_item',
                       pid_value=CommunityPIDValue(
                           self['control_number'],
                           current_oarepo_communities.get_primary_community_field(self)),
                       _external=True)


class AllNResultRecord(SchemaKeepingRecordMixin, CommunityRecordMixin, Record):
    ALLOWED_SCHEMAS = NRESULTS_ALLOWED_SCHEMAS
    PREFERRED_SCHEMA = NRESULTS_PREFERRED_SCHEMA
    index_name = all_nresults_index_name

    @property
    def canonical_url(self):
        if not self.get('oarepo:draft'):
            endpoint = 'invenio_records_rest.nresults-community_item'
        else:
            endpoint = 'invenio_records_rest.draft-nresults-community_item'

        return url_for(endpoint,
                       pid_value=CommunityPIDValue(
                           self['control_number'],
                           current_oarepo_communities.get_primary_community_field(self)),
                       _external=True)



import os

NRESULTS_ALLOWED_SCHEMAS = ['nr_nresults/nr-nresults-v1.0.0.json']
NRESULTS_PREFERRED_SCHEMA = 'nr_nresults/nr-nresults-v1.0.0.json'


DRAFT_NRESULT_PID_TYPE = 'dnrnrs'
DRAFT_NRESULT_RECORD = 'nr_nresults.record:DraftNResultRecord'

PUBLISHED_NRESULT_PID_TYPE = 'nrnrs'
PUBLISHED_NRESULT_RECORD = 'nr_nresults.record:PublishedNResultRecord'

ALL_NRESULTS_PID_TYPE = 'anrnrs'
ALL_NRESULTS_RECORD_CLASS = 'nr_nresults.record:AllNResultRecord'


published_index_name = 'nr_nresults-nr-nresults-v1.0.0'
draft_index_name = 'draft-nr_nresults-nr-nresults-v1.0.0'
all_nresults_index_name = 'nr-all-nresults'


prefixed_published_index_name = os.environ.get('INVENIO_SEARCH_INDEX_PREFIX',
                                               '') + published_index_name
prefixed_draft_index_name = os.environ.get('INVENIO_SEARCH_INDEX_PREFIX', '') + draft_index_name
prefixed_all_nresults_index_name = os.environ.get('INVENIO_SEARCH_INDEX_PREFIX', '') + all_nresults_index_name



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 CIS UCT Prague.
#
# CIS theses repository is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Default configuration."""

from __future__ import absolute_import, print_function

from functools import partial

from invenio_records_rest.utils import allow_all, deny_all
from nr_common.search import community_search_factory
from oarepo_communities.links import community_record_links_factory

from nr_nresults.constants import PUBLISHED_NRESULT_PID_TYPE, PUBLISHED_NRESULT_RECORD, DRAFT_NRESULT_PID_TYPE, \
    DRAFT_NRESULT_RECORD, ALL_NRESULTS_RECORD_CLASS, ALL_NRESULTS_PID_TYPE, all_nresults_index_name
from nr_nresults.record import draft_index_name, published_index_name
from nr_nresults.search import NResultsRecordsSearch
from nr_common.links import nr_links_factory

RECORDS_DRAFT_ENDPOINTS = {
    'nresults-community': {
        'draft': 'draft-nresults-community',
        'pid_type': PUBLISHED_NRESULT_PID_TYPE,
        'pid_minter': 'nr_nresults',
        'pid_fetcher': 'nr_nresults',
        'default_endpoint_prefix': True,
        'max_result_window': 500000,
        'record_class': PUBLISHED_NRESULT_RECORD,
        'search_index': published_index_name,
        'search_factory_imp': community_search_factory,

        'list_route': '/<community_id>/nresults/',
        'item_route': f'/<commpid(nrthe,model="nresults",record_class="nr_nresults.record:PublishedNresultRecord"):pid_value>',

        'publish_permission_factory_imp': 'nr_common.permissions.publish_draft_object_permission_impl',
        'unpublish_permission_factory_imp': 'nr_common.permissions.unpublish_draft_object_permission_impl',
        'edit_permission_factory_imp': 'nr_common.permissions.update_object_permission_impl',
        'list_permission_factory_imp': allow_all,
        'read_permission_factory_imp': allow_all,
        'create_permission_factory_imp': deny_all,
        'update_permission_factory_imp': deny_all,
        'delete_permission_factory_imp': deny_all,
        'default_media_type': 'application/json',
        'links_factory_imp': partial(community_record_links_factory, original_links_factory=nr_links_factory),
        'search_class': NResultsRecordsSearch,
        # 'indexer_class': CommitingRecordIndexer,
        'files': dict(
            # Who can upload attachments to a draft dataset record
            put_file_factory=deny_all,
            # Who can download attachments from a draft dataset record
            get_file_factory=allow_all,
            # Who can delete attachments from a draft dataset record
            delete_file_factory=deny_all
        )

    },
    'draft-nresults-community': {
        'pid_type': DRAFT_NRESULT_PID_TYPE,
        'record_class': DRAFT_NRESULT_RECORD,

        'list_route': '/<community_id>/nresults/draft/',
        'item_route': f'/<commpid(nrthe,model="nresults/draft",record_class="nr_nresults.record:DraftNresultRecord"):pid_value>',
        'search_index': draft_index_name,
        'links_factory_imp': partial(community_record_links_factory, original_links_factory=nr_links_factory),
        'search_factory_imp': community_search_factory,
        'search_class': NResultsRecordsSearch,
        'search_serializers': {
            'application/json': 'oarepo_validate:json_search',
        },
        'record_serializers': {
            'application/json': 'oarepo_validate:json_response',
        },

        'create_permission_factory_imp': 'nr_common.permissions.create_draft_object_permission_impl',
        'update_permission_factory_imp': 'nr_common.permissions.update_draft_object_permission_impl',
        'read_permission_factory_imp': 'nr_common.permissions.read_draft_object_permission_impl',
        'delete_permission_factory_imp': 'nr_common.permissions.delete_draft_object_permission_impl',
        'list_permission_factory_imp': 'nr_common.permissions.list_draft_object_permission_impl',
        'record_loaders': {
            'application/json': 'oarepo_validate.json_files_loader',
            'application/json-patch+json': 'oarepo_validate.json_loader'
        },
        'files': dict(
            put_file_factory='nr_common.permissions.put_draft_file_permission_impl',
            get_file_factory='nr_common.permissions.get_draft_file_permission_impl',
            delete_file_factory='nr_common.permissions.delete_draft_file_permission_impl'
        )

    },
    'nresults': {
        'draft': 'draft-nresults',
        'pid_type': PUBLISHED_NRESULT_PID_TYPE + '-nresults',
        'pid_minter': 'nr_nresults',
        'pid_fetcher': 'nr_nresults',
        'default_endpoint_prefix': True,
        'max_result_window': 500000,
        'record_class': ALL_NRESULTS_RECORD_CLASS,
        'search_index': published_index_name,

        'list_route': '/nresults/',
        'item_route': f'/not-really-used',
        'publish_permission_factory_imp': deny_all,
        'unpublish_permission_factory_imp': deny_all,
        'edit_permission_factory_imp': deny_all,
        'list_permission_factory_imp': allow_all,
        'read_permission_factory_imp': allow_all,
        'create_permission_factory_imp': deny_all,
        'update_permission_factory_imp': deny_all,
        'delete_permission_factory_imp': deny_all,
        'default_media_type': 'application/json',
        'links_factory_imp': partial(community_record_links_factory, original_links_factory=nr_links_factory),
        'search_class': NResultsRecordsSearch,
        # 'indexer_class': CommitingRecordIndexer,
        'files': dict(
            # Who can upload attachments to a draft dataset record
            put_file_factory=deny_all,
            # Who can download attachments from a draft dataset record
            get_file_factory=allow_all,
            # Who can delete attachments from a draft dataset record
            delete_file_factory=deny_all
        )
    },
    'draft-nresults': {
        'pid_type': DRAFT_NRESULT_PID_TYPE + '-draft-nresults',
        'record_class': ALL_NRESULTS_RECORD_CLASS,

        'list_route': '/nresults/draft/',
        'item_route': f'/not-really-used',
        'search_index': draft_index_name,
        'links_factory_imp': partial(community_record_links_factory, original_links_factory=nr_links_factory),
        'search_class': NResultsRecordsSearch,
        'search_serializers': {
            'application/json': 'oarepo_validate:json_search',
        },
        'record_serializers': {
            'application/json': 'oarepo_validate:json_response',
        },

        'create_permission_factory_imp': deny_all,
        'update_permission_factory_imp': deny_all,
        'read_permission_factory_imp': 'nr_common.permissions.read_draft_object_permission_impl',
        'delete_permission_factory_imp': deny_all,
        'list_permission_factory_imp': 'nr_common.permissions.list_draft_object_permission_impl',
        'files': dict(
            put_file_factory=deny_all,
            get_file_factory='nr_common.permissions.get_draft_file_permission_impl',
            delete_file_factory=deny_all
        )
    }
}


RECORDS_REST_ENDPOINTS = {
    'all-nresults': dict(
        pid_type=ALL_NRESULTS_PID_TYPE,
        pid_minter='nr_all',
        pid_fetcher='nr_all',
        default_endpoint_prefix=True,
        record_class=ALL_NRESULTS_RECORD_CLASS,
        search_class=NResultsRecordsSearch,
        search_index=all_nresults_index_name,
        search_serializers={
            'application/json': 'oarepo_validate:json_search',
        },
        list_route='/nresults/all/',
        links_factory_imp=partial(community_record_links_factory, original_links_factory=nr_links_factory),
        default_media_type='application/json',
        max_result_window=10000,
        # not used really
        item_route=f'/nresults/'
                   f'/not-used-but-must-be-present',
        list_permission_factory_imp='nr_common.permissions.list_all_object_permission_impl',
        create_permission_factory_imp=deny_all,
        delete_permission_factory_imp=deny_all,
        update_permission_factory_imp=deny_all,
        read_permission_factory_imp=deny_all,
        record_serializers={
            'application/json': 'oarepo_validate:json_response',
        },
        use_options_view=False
    ),
    'community-nresults': dict(
        pid_type=ALL_NRESULTS_PID_TYPE + '-community-all',
        pid_minter='nr_all',
        pid_fetcher='nr_all',
        default_endpoint_prefix=True,
        record_class=ALL_NRESULTS_RECORD_CLASS,
        search_class=NResultsRecordsSearch,
        search_index=all_nresults_index_name,
        search_factory_imp=community_search_factory,
        search_serializers={
            'application/json': 'oarepo_validate:json_search',
        },
        list_route='/<community_id>/nresults/all/',
        links_factory_imp=partial(community_record_links_factory, original_links_factory=nr_links_factory),
        default_media_type='application/json',
        max_result_window=10000,
        # not used really
        item_route=f'/nresult/'
                   f'/not-used-but-must-be-present',
        list_permission_factory_imp='nr_common.permissions.list_all_object_permission_impl',
        create_permission_factory_imp=deny_all,
        delete_permission_factory_imp=deny_all,
        update_permission_factory_imp=deny_all,
        read_permission_factory_imp=deny_all,
        record_serializers={
            'application/json': 'oarepo_validate:json_response',
        },
        use_options_view=False
    )
}

FILTERS = {
}

POST_FILTERS = {
}

RECORDS_REST_FACETS = {
}

RECORDS_REST_SORT_OPTIONS = {
}

RECORDS_REST_DEFAULT_SORT = {
}

"""Set default sorting options."""



import logging

from . import config

log = logging.getLogger('nr-events')


class NRNresults(object):
    """CIS theses repository extension."""

    def __init__(self, app=None):
        """Extension initialization."""
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Flask application initialization."""
        self.init_config(app)

    def init_config(self, app):
        """Initialize configuration.

        Override configuration variables with the values in this package.
        """
        app.config.setdefault('RECORDS_DRAFT_ENDPOINTS', {}).update(config.RECORDS_DRAFT_ENDPOINTS)
        app.config.setdefault('RECORDS_REST_ENDPOINTS', {}).update(config.RECORDS_REST_ENDPOINTS)
        app.config.setdefault('RECORDS_REST_FACETS', {}).update(config.RECORDS_REST_FACETS)

        app.config.setdefault('RECORDS_REST_SORT_OPTIONS', {}).update(
            config.RECORDS_REST_SORT_OPTIONS)

        app.config.setdefault('RECORDS_REST_DEFAULT_SORT', {}).update(
            config.RECORDS_REST_DEFAULT_SORT)



from invenio_search import RecordsSearch
from nr_common.search import NRRecordsSearch


class NResultsRecordsSearch(NRRecordsSearch):
    LIST_SOURCE_FIELDS = [
        'control_number', 'oarepo:validity.valid', 'oarepo:draft', 'title', 'dateIssued',
        'creator', 'resourceType', 'contributor', 'keywords', 'subject', 'abstract',
        'state',
        '_administration.primaryCommunity',
        '_administration.communities'
    ]
    HIGHLIGHT_FIELDS = {
        'title.cs': None,
        'title._': None,
        'title.en': None
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 CERN.
#
# My site is free software; you can redistribute it and/or modify it under
# the terms of the MIT License; see LICENSE file for more details.

"""Mappings.

Mappings define how nr-nresults and their fields will be indexed in Elasticsearch.
The provided nusl-common-v1.0.0.json file is an example of how to index nr-nresults
in Elasticsearch. You need to provide one mapping per major version of
Elasticsearch you want to support.
"""

from __future__ import absolute_import, print_function



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 CERN.
#
# My site is free software; you can redistribute it and/or modify it under
# the terms of the MIT License; see LICENSE file for more details.

"""JSON Schemas."""
from nr_common.marshmallow.fields import DateString
from nr_common.marshmallow.json import CommonMetadataSchemaV2
from nr_common.marshmallow.subschemas import TitledMixin
from invenio_records_rest.schemas.fields import SanitizedUnicode
from marshmallow.validate import Length
from oarepo_taxonomies.marshmallow import TaxonomyField

from nr_nresults.marshmallow.subschemas import NcertifyingAuthorityMixin


class NResultsMetadataSchemaV1(CommonMetadataSchemaV2):
    N_certifyingAuthority = TaxonomyField(mixins=[TitledMixin, NcertifyingAuthorityMixin])
    N_dateCertified = DateString()
    N_economicalParameters = SanitizedUnicode(validate=Length(max=1024))
    N_internalID = SanitizedUnicode()
    N_referenceNumber = SanitizedUnicode()
    N_resultUsage = TaxonomyField(mixins=[TitledMixin])
    N_technicalParameters = SanitizedUnicode(validate=Length(max=3000))
    N_type = TaxonomyField(mixins=[TitledMixin], required=True)
